@@ -22,7 +22,7 @@ logger = logging.getLogger("nautobot.authentication")
 class ObjectPermissionBackend(ModelBackend):
     def get_all_permissions(self, user_obj, obj=None):
         if not user_obj.is_active or user_obj.is_anonymous:
-            return dict()
+            return {}
         if not hasattr(user_obj, "_object_perm_cache"):
             user_obj._object_perm_cache = self.get_object_permissions(user_obj)
         return user_obj._object_perm_cache
@@ -32,6 +32,7 @@ class ObjectPermissionBackend(ModelBackend):
         Return all permissions granted to the user by an ObjectPermission.
         """
         # Retrieve all assigned and enabled ObjectPermissions
+        # v2 TODO(jathan): Replace prefetch_related with select_related
         object_permissions = ObjectPermission.objects.filter(
             Q(users=user_obj) | Q(groups__user=user_obj), enabled=True
         ).prefetch_related("object_types")
@@ -48,9 +49,9 @@ class ObjectPermissionBackend(ModelBackend):
 
     def has_perm(self, user_obj, perm, obj=None):
         if perm == "is_staff":
-            return user_obj.is_active and (user_obj.obj.is_staff or user_obj.is_superuser)
+            return user_obj.is_active and (user_obj.is_staff or user_obj.is_superuser)
 
-        app_label, action, model_name = resolve_permission(perm)
+        app_label, _action, model_name = resolve_permission(perm)
 
         if app_label == "users" and model_name == "admingroup":
             perm = perm.replace("users", "auth").replace("admingroup", "group")
@@ -144,7 +145,7 @@ def assign_permissions_to_user(user, permissions=None):
     for permission_name, constraints in permissions.items():
         try:
             object_type, action = resolve_permission_ct(permission_name)
-            # TODO: Merge multiple actions into a single ObjectPermission per content type
+            # 2.0 TODO(jathan): Merge multiple actions into a single ObjectPermission per content type OR just replace this with a different solution entirely.
             obj_perm = ObjectPermission(name=permission_name, actions=[action], constraints=constraints)
             obj_perm.save()
             obj_perm.users.add(user)
