@@ -239,11 +239,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
     @classmethod
     def setUpTestData(cls):
 
-        cls.sites = (
-            Site.objects.create(name="Site 1", slug="site-1"),
-            Site.objects.create(name="Site 2", slug="site-2"),
-            Site.objects.create(name="Site 3", slug="site-3"),
-        )
+        cls.sites = Site.objects.all()[:3]
 
         statuses = Status.objects.get_for_model(Prefix)
 
@@ -278,7 +274,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"site__name": "Site 1"},
+            constraints={"site__name": self.sites[0].name},
             actions=["view"],
         )
         obj_perm.users.add(self.user)
@@ -305,7 +301,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"site__name": "Site 1"},
+            constraints={"site__name": self.sites[0].name},
             actions=["view"],
         )
         obj_perm.users.add(self.user)
@@ -314,7 +310,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Retrieve all objects. Only permitted objects should be returned.
         response = self.client.get(url, **self.header)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.data["count"], 3)
+        self.assertEqual(response.data["count"], Prefix.objects.filter(site=self.sites[0]).count())
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
     def test_create_object(self):
@@ -333,7 +329,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"site__name": "Site 1"},
+            constraints={"site__name": self.sites[0].name},
             actions=["add"],
         )
         obj_perm.users.add(self.user)
@@ -362,7 +358,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"site__name": "Site 1"},
+            constraints={"site__name": f"{self.sites[0].name}"},
             actions=["change"],
         )
         obj_perm.users.add(self.user)
@@ -397,7 +393,7 @@ class ObjectPermissionAPIViewTestCase(TestCase):
         # Assign object permission
         obj_perm = ObjectPermission.objects.create(
             name="Test permission",
-            constraints={"site__name": "Site 1"},
+            constraints={"site__name": self.sites[0].name},
             actions=["delete"],
         )
         obj_perm.users.add(self.user)
