@@ -7,7 +7,7 @@ from packaging import version
 
 from django.core.exceptions import ValidationError
 from django.template.loader import get_template
-from django.urls import URLPattern
+from django.urls import get_resolver, URLPattern
 
 from nautobot.core.apps import (
     NautobotConfig,
@@ -24,6 +24,7 @@ from nautobot.extras.plugins.exceptions import PluginImproperlyConfigured
 from nautobot.extras.plugins.utils import import_object
 from nautobot.extras.secrets import register_secrets_provider
 from nautobot.utilities.choices import ButtonColorChoices
+from nautobot.utilities.deprecation import class_deprecated_in_favor_of
 
 logger = getLogger(__name__)
 
@@ -41,7 +42,7 @@ registry["plugin_template_extensions"] = collections.defaultdict(list)
 #
 
 
-class PluginConfig(NautobotConfig):
+class NautobotAppConfig(NautobotConfig):
     """
     Subclass of Django's built-in AppConfig class, to be used for Nautobot plugins.
     """
@@ -95,6 +96,7 @@ class PluginConfig(NautobotConfig):
     menu_items = "navigation.menu_items"
     secrets_providers = "secrets.secrets_providers"
     template_extensions = "template_content.template_extensions"
+    override_views = "views.override_views"
 
     def ready(self):
         """Callback after plugin app is loaded."""
@@ -191,6 +193,15 @@ class PluginConfig(NautobotConfig):
                         f"{filter_extension.model} -> {filterform_field_name}"
                     )
 
+        # Register override view (if any)
+        override_views = import_object(f"{self.__module__}.{self.override_views}")
+        if override_views is not None:
+            for qualified_view_name, view in override_views.items():
+                self.features.setdefault("overridden_views", []).append(
+                    (qualified_view_name, f"{view.__module__}.{view.__name__}")
+                )
+            register_override_views(override_views, self.name)
+
     @classmethod
     def validate(cls, user_config, nautobot_version):
         """Validate the user_config for baseline correctness."""
@@ -242,18 +253,23 @@ class PluginConfig(NautobotConfig):
                 user_config[setting] = value
 
 
+@class_deprecated_in_favor_of(NautobotAppConfig)
+class PluginConfig(NautobotAppConfig):
+    pass
+
+
 #
 # Template content injection
 #
 
 
-class PluginTemplateExtension:
+class TemplateExtension:
     """
     This class is used to register plugin content to be injected into core Nautobot templates. It contains methods
     that are overridden by plugin authors to return template content.
 
     The `model` attribute on the class defines the which model detail page this class renders content for. It
-    should be set as a string in the form '<app_label>.<model_name>'. render() provides the following context data:
+    should be set as a string in the form `<app_label>.<model_name>`. `render()` provides the following context data:
 
     * object - The object being viewed
     * request - The current request
@@ -307,24 +323,50 @@ class PluginTemplateExtension:
         """
         raise NotImplementedError
 
+    def detail_tabs(self):
+        """
+        Tabs that will be rendered and added to the existing list of tabs on the detail page view.
+        Tabs will be ordered by their position in the list.
+
+        Content should be returned as a list of dicts in the following format:
+        ```
+        [
+            {
+                "title": "<title>",
+                "url": "<url for the tab link>",
+            },
+            {
+                "title": "<title>",
+                "url": "<url for the tab link>",
+            },
+        ]
+        ```
+        """
+        raise NotImplementedError
+
+
+@class_deprecated_in_favor_of(TemplateExtension)
+class PluginTemplateExtension(TemplateExtension):
+    pass
+
 
 def register_template_extensions(class_list):
     """
-    Register a list of PluginTemplateExtension classes
+    Register a list of TemplateExtension classes
     """
     # Validation
     for template_extension in class_list:
         if not inspect.isclass(template_extension):
-            raise TypeError(f"PluginTemplateExtension class {template_extension} was passed as an instance!")
-        if not issubclass(template_extension, PluginTemplateExtension):
-            raise TypeError(f"{template_extension} is not a subclass of extras.plugins.PluginTemplateExtension!")
+            raise TypeError(f"TemplateExtension class {template_extension} was passed as an instance!")
+        if not issubclass(template_extension, TemplateExtension):
+            raise TypeError(f"{template_extension} is not a subclass of nautobot.apps.ui.TemplateExtension!")
         if template_extension.model is None:
-            raise TypeError(f"PluginTemplateExtension class {template_extension} does not define a valid model!")
+            raise TypeError(f"TemplateExtension class {template_extension} does not define a valid model!")
 
         registry["plugin_template_extensions"][template_extension.model].append(template_extension)
 
 
-class PluginBanner:
+class Banner:
     """Class that may be returned by a registered plugin_banners function."""
 
     def __init__(self, content, banner_class=BannerClassChoices.CLASS_INFO):
@@ -334,6 +376,11 @@ class PluginBanner:
         self.banner_class = banner_class
 
 
+@class_deprecated_in_favor_of(Banner)
+class PluginBanner(Banner):
+    pass
+
+
 def register_banner_function(function):
     """
     Register a function that may return a Banner object.
@@ -377,7 +424,7 @@ def register_jobs(class_list):
     # That is done in response to the `nautobot_database_ready` signal, see nautobot.extras.signals.refresh_job_models
 
 
-class PluginFilterExtension:
+class FilterExtension:
     """Class that may be returned by a registered Filter Extension function."""
 
     model = None
@@ -387,18 +434,23 @@ class PluginFilterExtension:
     filterform_fields = {}
 
 
+@class_deprecated_in_favor_of(FilterExtension)
+class PluginFilterExtension(FilterExtension):
+    pass
+
+
 def register_filter_extensions(filter_extensions, plugin_name):
     """
-    Register a list of PluginFilterExtension classes
+    Register a list of FilterExtension classes
     """
     from nautobot.utilities.utils import get_filterset_for_model, get_form_for_model
     from nautobot.utilities.forms.utils import add_field_to_filter_form_class
 
     for filter_extension in filter_extensions:
-        if not issubclass(filter_extension, PluginFilterExtension):
-            raise TypeError(f"{filter_extension} is not a subclass of extras.plugins.PluginFilterExtension!")
+        if not issubclass(filter_extension, FilterExtension):
+            raise TypeError(f"{filter_extension} is not a subclass of nautobot.apps.filters.FilterExtension!")
         if filter_extension.model is None:
-            raise TypeError(f"PluginFilterExtension class {filter_extension} does not define a valid model!")
+            raise TypeError(f"FilterExtension class {filter_extension} does not define a valid model!")
 
         model_filterset_class = get_filterset_for_model(filter_extension.model)
         model_filterform_class = get_form_for_model(filter_extension.model, "Filter")
@@ -434,6 +486,8 @@ def register_filter_extensions(filter_extensions, plugin_name):
 #
 
 
+# 2.2 TODO: remove in favor of NavMenuItem
+@class_deprecated_in_favor_of(NavMenuItem)
 class PluginMenuItem:
     """
     This class represents a navigation menu item. This constitutes primary link and its text, but also allows for
@@ -459,6 +513,8 @@ class PluginMenuItem:
             self.buttons = buttons
 
 
+# 2.2 TODO: remove in favor of NavMenuButton
+@class_deprecated_in_favor_of(NavMenuButton)
 class PluginMenuButton:
     """
     This class represents a button within a PluginMenuItem. Note that button colors should come from
@@ -557,15 +613,15 @@ def register_plugin_menu_items(section_name, menu_items):
 #
 
 
-class PluginCustomValidator:
+class CustomValidator:
     """
     This class is used to register plugin custom model validators which act on specified models. It contains the clean
     method which is overridden by plugin authors to execute custom validation logic. Plugin authors must raise
-    ValidationError within this method to trigger validation error messages which are propgated to the user.
+    ValidationError within this method to trigger validation error messages which are propagated to the user.
     A convenience method `validation_error(<message>)` may be used for this purpose.
 
     The `model` attribute on the class defines the model to which this validator is registered. It
-    should be set as a string in the form '<app_label>.<model_name>'.
+    should be set as a string in the form `<app_label>.<model_name>`.
     """
 
     model = None
@@ -576,7 +632,7 @@ class PluginCustomValidator:
     def validation_error(self, message):
         """
         Convenience method for raising `django.core.exceptions.ValidationError` which is required in order to
-        trigger validation error messages which are propgated to the user.
+        trigger validation error messages which are propagated to the user.
         """
         raise ValidationError(message)
 
@@ -584,23 +640,58 @@ class PluginCustomValidator:
         """
         Implement custom model validation in the standard Django clean method pattern. The model instance is accessed
         with the `object` key within `self.context`, e.g. `self.context['object']`. ValidationError must be raised to
-        prevent saving model instance changes, and propogate messages to the user. For convenience,
+        prevent saving model instance changes, and propagate messages to the user. For convenience,
         `self.validation_error(<message>)` may be called to raise a ValidationError.
         """
         raise NotImplementedError
 
 
+@class_deprecated_in_favor_of(CustomValidator)
+class PluginCustomValidator(CustomValidator):
+    pass
+
+
 def register_custom_validators(class_list):
     """
-    Register a list of PluginCustomValidator classes
+    Register a list of CustomValidator classes
     """
     # Validation
     for custom_validator in class_list:
         if not inspect.isclass(custom_validator):
-            raise TypeError(f"PluginCustomValidator class {custom_validator} was passed as an instance!")
-        if not issubclass(custom_validator, PluginCustomValidator):
-            raise TypeError(f"{custom_validator} is not a subclass of extras.plugins.PluginCustomValidator!")
+            raise TypeError(f"CustomValidator class {custom_validator} was passed as an instance!")
+        if not issubclass(custom_validator, CustomValidator):
+            raise TypeError(f"{custom_validator} is not a subclass of nautobot.apps.models.CustomValidator!")
         if custom_validator.model is None:
-            raise TypeError(f"PluginCustomValidator class {custom_validator} does not define a valid model!")
+            raise TypeError(f"CustomValidator class {custom_validator} does not define a valid model!")
 
         registry["plugin_custom_validators"][custom_validator.model].append(custom_validator)
+
+
+def register_override_views(override_views, plugin):
+
+    validation_error = (
+        "Plugin '{}' tried to override view '{}' but did not contain a valid app name "
+        "(e.g. `dcim:device`, `plugins:myplugin:myview`)."
+    )
+
+    for qualified_view_name, view in override_views.items():
+        resolver = get_resolver()
+
+        try:
+            qualified_app_name, view_name = qualified_view_name.rsplit(":", 1)
+            app_resolver = resolver
+            for app_name in qualified_app_name.split(":"):
+                app_resolver_tupl = app_resolver.namespace_dict.get(app_name)
+                if app_resolver_tupl is None:
+                    # We couldn't find the app, regardless of nesting
+                    raise ValidationError(validation_error.format(plugin, qualified_view_name))
+
+                app_resolver = app_resolver_tupl[1]
+
+        except ValueError:
+            # This is only thrown when qualified_view_name does not contain ":"
+            raise ValidationError(validation_error.format(plugin, qualified_view_name))
+
+        for pattern in app_resolver.url_patterns:
+            if isinstance(pattern, URLPattern) and hasattr(pattern, "name") and pattern.name == view_name:
+                pattern.callback = view
