@@ -1,4 +1,5 @@
 import json
+import logging
 import os
 import tempfile
 from unittest import mock
@@ -20,7 +21,6 @@ from nautobot.extras.choices import (
 from nautobot.extras.choices import LogLevelChoices
 from nautobot.extras.datasources.git import pull_git_repository_and_refresh_data, git_repository_diff_origin_and_local
 from nautobot.extras.datasources.registry import get_datasource_contents
-from nautobot.extras.management import create_custom_statuses
 from nautobot.extras.models import (
     ConfigContext,
     ConfigContextSchema,
@@ -45,13 +45,10 @@ class GitTest(TransactionTestCase):
     """
 
     databases = ("default", "job_logs")
-
     COMMIT_HEXSHA = "88dd9cd78df89e887ee90a1d209a3e9a04e8c841"
 
     def setUp(self):
         super().setUp()
-        # Repopulate custom statuses between test cases, as TransactionTestCase deletes them during cleanup
-        create_custom_statuses(None, verbosity=0)
 
         self.factory = RequestFactory()
         self.mock_request = self.factory.get("/no-op/")
@@ -117,7 +114,7 @@ class GitTest(TransactionTestCase):
 
     def populate_repo(self, path, url, *args, **kwargs):
         os.makedirs(path)
-        # TODO: populate Jobs as well?
+        # TODO(Glenn): populate Jobs as well?
         os.makedirs(os.path.join(path, "config_contexts"))
         os.makedirs(os.path.join(path, "config_contexts", "devices"))
         os.makedirs(os.path.join(path, "config_context_schemas"))
@@ -551,7 +548,9 @@ class GitTest(TransactionTestCase):
                 MockGitRepo.return_value.checkout.return_value = self.COMMIT_HEXSHA
 
                 # Run the Git operation and refresh the object from the DB
+                logging.disable(logging.ERROR)
                 pull_git_repository_and_refresh_data(self.repo.pk, self.mock_request, self.job_result.pk)
+                logging.disable(logging.NOTSET)
                 self.job_result.refresh_from_db()
 
                 self.assertEqual(
